package pgdp.trees;

import java.util.Comparator;
import java.util.function.Predicate;

public class InnerNode<T> implements TreeElement<T> {

	private T info;
	private TreeElement<T> left;
	private TreeElement<T> right;

	public InnerNode(T info) {
		this.info = info;
		// ein neuer innerer Knoten entsteht immer ganz unten und ist daher links und
		// rechts ein Blatt
		this.left = new Leaf<T>();
		this.right = new Leaf<T>();
	}

	@Override
	public TreeElement<T> insert(T value, Comparator<T> comp) {
		// anhand der Vergleichsfunktion prüfen, ob das Element links oder rechts des
		// aktuellen Knotens einsortiert werden muss
		if (comp.compare(value, this.info) < 0)
			this.left = this.left.insert(value, comp);
		else
			this.right = this.right.insert(value, comp);
		return this;
	}

	@Override
	public void toString(StringBuilder sb) {
		// linken Teilbaum hinzufügen
		this.left.toString(sb);

		// eventuell Komma hinzufügen
		if (sb.length() != 0)
			sb.append(",");

		// aktuellen Knoten hinzufügen
		sb.append(this.info);

		// rechten Teilbaum hinzufügen
		this.right.toString(sb);

	}

	@Override
	public int size() {
		// 1 (dieser Knoten) + länge links/rechts
		return 1 + this.left.size() + this.right.size();
	}

	@Override
	public T getMin() {
		// laut Invariante ist der innere Knoten ganz unten Links der minimale
		if (this.left instanceof Leaf)
			return this.info;
		return this.left.getMin();
	}

	@Override
	public TreeElement<T> remove(T value, Comparator<T> comp) {
		// linker oder rechter Teilbaum
		// Kinder werden aktualisiert, falls removed wird
		if (comp.compare(value, this.info) < 0)
			this.left = this.left.remove(value, comp);
		else if (comp.compare(value, this.info) > 0)
			this.right = this.right.remove(value, comp);
		else {
			// das gesuchte Element gefunden (compare ergibt 0; das Gleiche)

			// falls rechts ein Blatt ist, können wir den rechten Teilbaum ignorieren und
			if (this.right instanceof Leaf)
				return left;

			// kleinstes Element im rechten Teilbaum (= das nächst größere Element) wird
			// neue Wurzel des Teilbaums > damit bleibt invariante bestehen
			this.info = this.right.getMin();

			// Element soll nicht doppelt vorkommen > wird im rechten Teilbaum gelöscht
			this.right = this.remove(this.info, comp);
		}

		// in dem durchgang wurde nichts verändert, also an Eltern keine Veränderung
		// weiterreichen
		return this;
	}

	@Override
	public boolean contains(T value, Comparator<T> comp) {
		// prüfen, welche Seite des Baumes durchsucht werden muss, falls comp = 0, hat
		// man das Element gefunden

		// prüfen, ob man nach rechts muss
		if (comp.compare(value, this.info) < 0)
			return this.right.contains(value, comp);

		// prüfen ob man nach links muss
		if (comp.compare(value, info) > 0)
			return this.left.contains(value, comp);

		// falls man hier ankommt, liefert comp 0 zurück, also Element das Gleiche
		return true;

	}

	@Override
	public int countMatches(Predicate<T> filter) {
		// gucken ob aktueller Knoten die Bedingung erfüllt, danach links und rechts
		// prüfen
		int sum = 0;
		if (filter.test(this.info))
			sum = sum + 1;

		// links
		sum = sum + this.left.countMatches(filter);

		// rechts
		sum = sum + this.right.countMatches(filter);

		return sum;
	}

	@Override
	public int getAll(Predicate<T> filter, T[] array, int index) {
		// links suchen (da sind die kleineren Elemente
		index = this.left.getAll(filter, array, index);

		// aktueller Knoten
		if (filter.test(this.info))
			array[index++] = info;

		// rechts suchen
		index = this.right.getAll(filter, array, index);

		return index;
	}

}
